@@ -1,9 +1,33 @@
+# version 1.5.5 February 12, 2026 by alexis.dinno@pdx.edu
+# perform Horn's parallel analysis for a principal component or common factor 
+# analysis, so as to adjust for finite sample bias in the retention of 
+# components.
+
 `paran` <-
-function(x=NA, iterations=0, centile=0, quietly=FALSE, status=TRUE, all=FALSE, cfa=FALSE, graph=FALSE, color=TRUE, col=c("black","red","blue"), lty=c(1,2,3), lwd=1, legend=TRUE, file="", width=640, height=640, grdevice="png", seed=0, mat=NA, n=NA) {
+function(x=NA, 
+    iterations=0,
+    centile=0,
+    quietly=FALSE,
+    status=TRUE,
+    all=FALSE,
+    cfa=FALSE,
+    graph=FALSE,
+    color=TRUE,
+    col=c("black", "red", "blue"),
+    lty=c(1,2,3),
+    lwd=1,
+    legend=TRUE,
+    file="",
+    width=640,
+    height=640,
+    grdevice="png",
+    seed=0,
+    mat=NA,
+    n=NA) {
 
 # Confirm that x and mat were NOT both provided
   if ( !is.na(mat[[1]][1]) & !is.na(x[[1]][1]) ) {
-    stop("\nYou must supply either x or mat but not both.")
+    rlang::abort(message="You must supply either x or mat but not both.")
     }
 
 # Set the RNG seed if necessary
@@ -22,36 +46,45 @@ function(x=NA, iterations=0, centile=0, quietly=FALSE, status=TRUE, all=FALSE, c
 # Confirm correlation matrix, and not covariance matrix
   if (!is.na(mat[[1]][1])) {
     if (length(mat[1,]) != length(mat[,1])) {
-      stop("\nThe matrix provided with the mat argument is not a correlation matrix.\n")
+      rlang::abort(message="The matrix provided with the mat argument is not a correlation matrix.")
       }
     if ( is.element(FALSE , (diag(mat) == rep(1,P))) ) {
-      stop("\nThe matrix provided with the mat argument is not a correlation matrix.\nParallel analysis is not compatible with the eigendecomposition of a covariance matrix.")
+      rlang::abort(message="The matrix provided with the mat argument is not a correlation matrix.\nParallel analysis is not compatible with the eigendecomposition of a covariance matrix.")
       }
     }
 
+# Is message suppression requested?
+  muffled <- TRUE %in% grepl("muffleMessage", deparse(sys.call(-1)), fixed = TRUE)
+
+# Is global option rlib_message_verbosity quiet?
+  if (is.null(getOption("rlib_message_verbosity")) == FALSE) {
+    if (getOption("rlib_message_verbosity") == "quiet") quietly <- TRUE
+    }
+  if (muffled == TRUE) quietly <- TRUE
+  
 # Make the correlation matrix R from the data, or take
 # from the supplied matrix
   if ( !is.na(x[[1]][1]) ) {
     N <- length(as.matrix(x)[,1])
     if ( !is.na(n) ) {
-      warning("\nThe n argument is only for use with the mat argument. Ignoring n argument.")
+      rlang::warn(message="The n argument is only for use with the mat argument. Ignoring n argument.")
       }
     R <- cor(x)
-    cat("\nUsing eigendecomposition of correlation matrix.")
+    if (quietly == FALSE) rlang::inform(message="Using eigendecomposition of correlation matrix.")
     }
   if ( !is.na(mat[[1]][1]) ) {
     if (!is.na(mat[[1]][1]) & is.na(n)) {
-      stop("\nYou must also provide the sample size when using the matrix argument.")
+      rlang::abort(message="You must also provide the sample size when using the matrix argument.")
       }
     N <- n
     R <- mat
-    cat("\nUsing eigendecomposition of provided correlation matrix.")
+    if (quietly == FALSE) rlang::inform(message="Using eigendecomposition of provided correlation matrix.")
     }
 
 # quick validation of centile as an integer value
   centile <- round(centile)
   if (centile > 99 | centile < 0) {
-    stop("\nYou must specify a centile value between 1 and 99.\n(Specifying centile 0 will use the mean.)")
+    rlang::abort(message="You must specify a centile value between 1 and 99.\n(Specifying centile 0 will use the mean.)")
     }
 
 # Perform pca or cfa
@@ -83,7 +116,7 @@ function(x=NA, iterations=0, centile=0, quietly=FALSE, status=TRUE, all=FALSE, c
     iterations <- 30*P
     }
    if (iterations<0) {
-    cat("\nInvalid number of iterations! Using default value of ",iterations,"\n",sep="")
+    rlang::inform(message=paste0("Invalid number of iterations! Using default value of ", iterations, sep=""))
     }
 
 # prepare to save the results of each pca
@@ -95,24 +128,22 @@ function(x=NA, iterations=0, centile=0, quietly=FALSE, status=TRUE, all=FALSE, c
       SimEvs <- matrix(NA,iterations,P)
       }
 
-# Let the user know the program is working if neccesary
-  if (status==TRUE) {
-    if (iterations >= 10) {
-      cat("\nComputing: ")
-      }
-    }
-
   for (k in 1:iterations) {
-   
-# Yet _more_ letting the user know the program is working!
-    if (status == TRUE) {
+  
+# Let the user know the program is working if neccesary
+    if (status == TRUE & quietly == FALSE) {
+      if (k == 1) cat("\nComputing: ")
+      progress <- ""
       if (k %% (iterations/10) == 1 & iterations >= 10 & k > 1) {
         pct <- (k%/%(iterations/10))*10
-        cat(pct,"%  ",sep="")
+        progress <- paste0(progress, pct,"%  ",sep="")
+        cat(progress)
         }
       if (k == iterations) {
-        cat("100%\n")
+        progress <- paste0(progress, "100%\n", sep="")
+        cat(progress)
         }
+      if (k == iterations) cat("\n")
       }
 
 # initialize previously created random dataset.
@@ -143,39 +174,42 @@ function(x=NA, iterations=0, centile=0, quietly=FALSE, status=TRUE, all=FALSE, c
 # end the for k loop
   }
 
-# display if neccesary
-  if (quietly == TRUE) {
-    cat("\n")
-    }
-  if (quietly == FALSE) {
-  
-    cat("\n\nResults of Horn's Parallel Analysis for ",model," retention\n",sep="")
+# Define the table horizontal border
+  table_header_footer <- paste0(paste0(rep("\U2500", times=50), collapse=""), sep="")
 
+# display if neccesary
+  if (quietly == FALSE) {
+    rlang::inform(message=paste0("\nResults of Horn's Parallel Analysis for ", model, " retention", sep=""))
     if (iterations == 1) {
+      mean_message <- "1 iteration, using the mean estimate\n"
+      centile_message <- paste0("1 iteration, using the ", centile, " centile estimate\n", sep="")
       if (centile == 0) {
-        cat("1 iteration, using the mean estimate","\n",sep="")
+        rlang::inform(message=mean_message)
         }
       if (centile != 0) {
-        cat("1 iteration, using the ",centile," centile estimate","\n",sep="")
+        rlang::inform(message=centile_message)
         }
       }
 
     if (iterations > 1) {
+      mean_message <- paste0(iterations, " iterations, using the mean estimate\n", sep="")
+      centile_message <- paste0(iterations, " iterations, using the ", centile, " centile estimate\n", sep="")
+      centile_median_message <- paste0(iterations, " iterations, using the ", centile, " centile (median) estimate", "\n", sep="")
       if (centile == 0) {
-        cat(iterations," iterations, using the mean estimate","\n",sep="")
+        rlang::inform(message=mean_message)
         }
       if (centile != 0 & centile != 50) {
-        cat(iterations," iterations, using the ",centile," centile estimate","\n",sep="")
+        rlang::inform(message=centile_message)
         }
       if (centile == 50) {
-        cat(iterations," iterations, using the ",centile," centile (median) estimate","\n",sep="")
+        rlang::inform(message=centile_median_message)
         }    
       }
-
-    cat("\n--------------------------------------------------","\n")
-    cat(Model,"  Adjusted    Unadjusted    Estimated","\n")
-    cat("            Eigenvalue  Eigenvalue    Bias","\n")
-    cat("--------------------------------------------------","\n")
+    table_header_footer <- paste0(paste0(rep("\U2500", times=50), collapse=""), sep="")
+    rlang::inform(message=table_header_footer)
+    rlang::inform(message=paste0(Model, pad.spaces(3), "Adjusted", pad.spaces(4), "Unadjusted", pad.spaces(4), "Estimated", sep=""))
+    rlang::inform(message=paste0(pad.spaces(12), "Eigenvalue", pad.spaces(2), "Eigenvalue", pad.spaces(4), "Bias", sep=""))
+    rlang::inform(message=table_header_footer)
     }
 
   RndEv = c(1:P)*NA 
@@ -192,9 +226,8 @@ function(x=NA, iterations=0, centile=0, quietly=FALSE, status=TRUE, all=FALSE, c
 
   if (Ev[[1]] < 1 | RndEv[[1]] < 1) { 
     if (quietly == FALSE) {
-      cat("No components passed.","\n")
-      cat("--------------------------------------------------","\n")
-      stop
+      rlang::inform(message="No components passed.","\n")
+      rlang::inform(message=table_header_footer)
       }
     }
 
@@ -321,17 +354,19 @@ function(x=NA, iterations=0, centile=0, quietly=FALSE, status=TRUE, all=FALSE, c
       }
 
     if (quietly == FALSE) {
-      cat(x,xPad,"      ",AdjFPad,AdjSpace,strtrim(AdjEv[x],SN),EvFPad,EvSpace,strtrim(Ev[[x]],EvSN),EvZPad,"     ",BiasSpace,strtrim(Bias[x],BiasSN),"\n", sep="")
+      eigenvalues_output_line = paste0(x, xPad, pad.spaces(6), AdjFPad, AdjSpace, strtrim(AdjEv[x], SN), EvFPad, EvSpace, strtrim(Ev[[x]], EvSN), EvZPad, pad.spaces(5), BiasSpace, strtrim(Bias[x], BiasSN), sep="")
+      rlang::inform(message=eigenvalues_output_line)
       }
     }
   if (quietly == FALSE) {
-    cat("--------------------------------------------------","\n")
+    rlang::inform(message=table_header_footer)
     if (cfa == TRUE) {
-      cat("\nAdjusted eigenvalues > 0 indicate dimensions to retain.\n(",retained," ",models," retained)\n\n",sep="")
+      retention_message <- paste0("Criterion: retain adjusted eigenvalues > 0.\n(", retained, " ", trimws(if (retained>1) models else model), " retained)\n", sep="")
       }
     if (cfa == FALSE) {
-      cat("\nAdjusted eigenvalues > 1 indicate dimensions to retain.\n(",retained," ",models," retained)\n\n",sep="")
+      retention_message <- paste0("Criterion: retain adjusted eigenvalues > 1.\n(", retained, " ", trimws(if (retained>1) models else model), " retained)\n", sep="")
       }
+    rlang::inform(message=retention_message)
     }
 
 # Graph it if needed
